#property strict

#property indicator_chart_window
#property indicator_buffers 9

extern int  ChannelPeriod = 52;
extern int  HighLowShift  = 26;
extern bool ShowMiddle    = true;
extern int  history = 10000;

extern color High_line_color_up = clrMagenta;
extern color High_line_color_dn = clrWhite;

extern color Low_line_color_up = clrMagenta;
extern color Low_line_color_dn = clrWhite;

extern color Middle_line_color_up = clrMagenta;
extern color Middle_line_color_dn = clrWhite;

extern ENUM_LINE_STYLE High_line_style = STYLE_SOLID;
extern ENUM_LINE_STYLE Low_line_style = STYLE_SOLID;
extern ENUM_LINE_STYLE Middle_line_style = STYLE_SOLID;

double buff_up[];
double buff_md[];
double buff_dn[];

double buff_up_add_1[];
double buff_up_add_2[];

double buff_dn_add_1[];
double buff_dn_add_2[];

double buff_md_add_1[];
double buff_md_add_2[];

int OnInit()
{   
   int line = 0;
   
   SetIndexBuffer(line, buff_up);
   SetIndexStyle(line, DRAW_LINE, High_line_style, EMPTY, High_line_color_up);
   ++line;
   
   SetIndexBuffer(line, buff_up_add_1);
   SetIndexStyle(line, DRAW_LINE, High_line_style, EMPTY, High_line_color_dn);
   ++line;
   
   SetIndexBuffer(line, buff_up_add_2);
   SetIndexStyle(line, DRAW_LINE, High_line_style, EMPTY, High_line_color_dn);
   ++line;

   SetIndexBuffer(line, buff_dn);
   SetIndexStyle(line, DRAW_LINE, Low_line_style, EMPTY, Low_line_color_up);
   ++line;
   
   SetIndexBuffer(line, buff_dn_add_1);
   SetIndexStyle(line, DRAW_LINE, Low_line_style, EMPTY, Low_line_color_dn);
   ++line;
   
   SetIndexBuffer(line, buff_dn_add_2);
   SetIndexStyle(line, DRAW_LINE, Low_line_style, EMPTY, Low_line_color_dn);
   ++line;
   
   if (ShowMiddle){
      SetIndexBuffer(line, buff_md);
      SetIndexStyle(line, DRAW_LINE, Middle_line_style, EMPTY, Middle_line_color_up);
      ++line;
      
      SetIndexBuffer(line, buff_md_add_1);
      SetIndexStyle(line, DRAW_LINE, Middle_line_style, EMPTY, Middle_line_color_dn);
      ++line;
      
      SetIndexBuffer(line, buff_md_add_2);
      SetIndexStyle(line, DRAW_LINE, Middle_line_style, EMPTY, Middle_line_color_dn);
      ++line;
   }
   else{
      SetIndexBuffer(line, buff_md);
      SetIndexStyle(line, DRAW_NONE);
      ++line;
      
      SetIndexBuffer(line, buff_md_add_1);
      SetIndexStyle(line, DRAW_NONE);
      ++line;
      
      SetIndexBuffer(line, buff_md_add_2);
      SetIndexStyle(line, DRAW_NONE);
      ++line;
   }
     
   return INIT_SUCCEEDED;
}

int start()
{
   int limit = countbars(history, true, ChannelPeriod + MathAbs(HighLowShift) + 5);
   if (limit < 0)
      return 0;
      
   if (limit > 1){   
      ArrayInitialize(buff_up, EMPTY_VALUE);
      ArrayInitialize(buff_md, EMPTY_VALUE);
      ArrayInitialize(buff_dn, EMPTY_VALUE);
      ArrayInitialize(buff_up_add_1, EMPTY_VALUE);
      ArrayInitialize(buff_up_add_2, EMPTY_VALUE);
      ArrayInitialize(buff_dn_add_1, EMPTY_VALUE);
      ArrayInitialize(buff_dn_add_2, EMPTY_VALUE);
      ArrayInitialize(buff_md_add_1, EMPTY_VALUE);
      ArrayInitialize(buff_md_add_2, EMPTY_VALUE);
   }

   for (int i = limit; i >= 0; --i)
   {
      buff_up[i]  = NormalizeDouble(High[Highest(_Symbol, _Period, MODE_HIGH, ChannelPeriod, i + HighLowShift)], Digits);
      buff_dn[i]  = NormalizeDouble(Low[Lowest(_Symbol, _Period, MODE_LOW, ChannelPeriod, i + HighLowShift)], Digits);
      buff_md[i] = NormalizeDouble((buff_up[i] + buff_dn[i])/2.0, Digits);
   }            
   
   MarkReducing(buff_up, buff_up_add_1, buff_up_add_2, limit, 0, EMPTY_VALUE);
   MarkReducing(buff_dn, buff_dn_add_1, buff_dn_add_2, limit, 0, EMPTY_VALUE);
   MarkReducing(buff_md, buff_md_add_1, buff_md_add_2, limit, 0, EMPTY_VALUE);
   
   return(0);
}


//==================================================================================================================//   
// "The function returns the number of bars, for which the calculation will be done"
// returns -1 when it is not necessary or not possible to perform calculation
//==================================================================================================================//                              
int countbars(int    history_ = 0,  // the number of bars, for which the calculation will be carried out (0-the whole story)
              bool   work_on_any_tick_ = false, // true - work on tick, false - work on new bar.
              int    need_bars_ = 0)       //  the minimum number of bars for the calculation of unit
{   
//--- check for avoiding of the out of the range of the timeseries Time
    static bool first_run = true;
    if (first_run == true){                              
       datetime time_check[];
       if (CopyTime(_Symbol, _Period, 0, 2, time_check) != 2)
          return -1;
       first_run = false;
    }
 
//--- last mount of the bars and time of the open of the last bar
    static int pre_bars = 0;         
    static datetime bar_time = 0;
    
    int bars_count = Bars;
    
//--- event new tick    
    if (bars_count == pre_bars && bar_time == Time[0]){
       if (work_on_any_tick_ == true)
          return(0); 
       return(-1);
    }
  
//--- event new bar    
    if (bars_count == pre_bars + 1 && bar_time == Time[1]){
       pre_bars = bars_count;  
       bar_time = Time[0];
       return (1);
    }   

//--- Not enough bars 
    if (bars_count < history_ + need_bars_){
       if (bars_count < need_bars_ + 2){
          if (bars_count != pre_bars) 
             Print ("Not enough bars, need: ", need_bars_ - bars_count + 2);
          pre_bars = bars_count;
          return(-1);
       }
       else{
          pre_bars = bars_count;
          bar_time = Time[0];
          
          static bool one_using = true;
          if (one_using == true){
             Print ("Not enough bars, will be used bars: ", bars_count - need_bars_ - 1);
             one_using = false;
          }
          return (bars_count - need_bars_ - 1);
       }
    }
    
//--- save previous data
    pre_bars = bars_count;  
    bar_time = Time[0];   
    
    if (history_ == 0)
       return (bars_count - need_bars_ - 1);
    return (history_);
}



//+------------------------------------------------------------------+
//|    MarkReducing                                                              |
//+------------------------------------------------------------------+
void MarkReducing( 
      double& values[],       
      double& reducing1[],    
      double& reducing2[],   
      int startIndex,         
      int endIndex,           
      double emptyValueUsed)  
{
   int di = -1;
   if (endIndex > startIndex) di = 1;
   
   for (int i = startIndex; (i >= startIndex && i <= endIndex) || (i >= endIndex && i <= startIndex); i += di)
   {
      if (i + 1 >= Bars)
      {
         continue;
      }

      if (
            values[i]      == emptyValueUsed || 
            values[i + 1]  == emptyValueUsed ||
            values[i]      == EMPTY_VALUE    || 
            values[i + 1]  == EMPTY_VALUE
      )
      {
         continue;
      }
      
      if (di < 0) reducing1[i] = EMPTY_VALUE;
      if (di < 0) reducing2[i] = EMPTY_VALUE;
      
      if (values[i] < values[i + 1])
      {
         if (values[i + 1] < values[i + 2])
         {
            if (reducing1[i + 1] != EMPTY_VALUE) reducing1[i] = values[i];
            else                                reducing2[i] = values[i];
         }
         else
         {
            if (reducing2[i + 2] == EMPTY_VALUE) 
            {
               reducing2[i] = values[i];
               reducing2[i + 1] = values[i + 1];
            }
            else
            {
               reducing1[i] = values[i];
               reducing1[i + 1] = values[i + 1];
            }
         }
      }
      else if (values[i] == values[i + 1]){
         if (reducing1[i + 1] != EMPTY_VALUE)
            reducing1[i] = values[i];
         if (reducing2[i + 1] != EMPTY_VALUE)
            reducing2[i] = values[i];
      }
      else if (i == 0)
      {
         if (reducing1[i + 1] != EMPTY_VALUE && reducing1[i + 2] == EMPTY_VALUE)
         {
            reducing1[i + 1] = EMPTY_VALUE;
         }

         if (reducing2[i + 1] != EMPTY_VALUE && reducing2[i + 2] == EMPTY_VALUE)
         {
            reducing2[i + 1] = EMPTY_VALUE;
         }
      }
   }
}